from commandlib import Command, python


def deploy(project_path, name, version):
    git = Command("git").in_dir(project_path)
    version_file = project_path.joinpath("VERSION")
    old_version = version_file.bytes().decode("utf8")
    if version_file.bytes().decode("utf8") != version:
        project_path.joinpath("VERSION").write_text(version)
        git("add", "VERSION").run()
        git(
            "commit", "-m", "RELEASE: Version {0} -> {1}".format(old_version, version)
        ).run()
        git("push").run()
        git("tag", "-a", version, "-m", "Version {0}".format(version)).run()
        git("push", "origin", version).run()
    else:
        git("push").run()

    # Set __version__ variable in __init__.py, build sdist and put it back
    initpy = project_path.joinpath(name, "__init__.py")
    original_initpy_contents = initpy.bytes().decode("utf8")
    initpy.write_text(original_initpy_contents.replace("DEVELOPMENT_VERSION", version))
    python("setup.py", "sdist").in_dir(project_path).run()
    initpy.write_text(original_initpy_contents)

    # Upload to pypi
    python("-m", "twine", "upload", "dist/{0}-{1}.tar.gz".format(name, version)).in_dir(
        project_path
    ).run()



import hitchbuildpy
import hitchrunpy
import hitchbuild
from copy import copy


class PyLibraryBuild(hitchbuild.HitchBuild):
    def __init__(self, project_name, paths):
        self._project_name = project_name
        self._paths = paths
        self._python_version = "3.7.0"
        self._packages = {}

    @property
    def pyenv(self):
        return hitchbuildpy.PyenvBuild(
            self._paths.share / "python{}".format(self._python_version),
            self._python_version
        )

    @property
    def virtualenv(self):
        vb = hitchbuildpy.VirtualenvBuild(
            build_path=self._paths.gen / "py{}".format(self._python_version),
            base_python=self.pyenv,
        )
        debugrequirements = self._paths.key.joinpath("debugrequirements.txt").abspath()
        if debugrequirements.exists():
            vb = vb.with_requirementstxt(debugrequirements)
        return vb

    @property
    def bin(self):
        return self.virtualenv.bin

    @property
    def working(self):
        return self._paths.gen / "working"

    def clean(self):
        self.virtualenv.clean()

    def with_python_version(self, python_version):
        new_build = copy(self)
        new_build._python_version = python_version
        return new_build

    def with_packages(self, packages):
        new_build = copy(self)
        new_build._packages = {**self._packages, **packages}
        return new_build

    @property
    def example_python_code(self):
        return hitchrunpy.ExamplePythonCode(
            self.bin.python,
            self._paths.gen,
        )

    def build(self):
        pipinstalle = self.virtualenv.incomplete()
        self.virtualenv.ensure_built()
        if pipinstalle:
            self.virtualenv.bin.pip("install", "-e", ".")\
                               .in_dir(self._paths.project)\
                               .run()
        
        


def project_build(project_name, paths, python_version, libraries=None):
    pylibrary = hitchbuildpy.VirtualenvBuild(
        base_python=hitchbuildpy.PyenvBuild(
            paths.share / "python{}".format(python_version), python_version
        ),
    ).with_requirementstxt(paths.key / "debugrequirements.txt")

    if libraries is not None:
        for library_name, library_version in libraries.items():
            pylibrary = pylibrary.with_packages(
                "{0}=={1}".format(library_name, library_version)
            )

    pylibrary.ensure_built()
    return pylibrary



import dirtemplate
from strictyaml import load
from git import Repo
import jinja2
from collections import OrderedDict

CHANGELOG_MD_TEMPLATE = """\
# Changelog

{% for version, changes in version_changes.items() %}
### {% if version != None %}{{ version }}{% else %}Latest{% endif %}

{% for change in changes -%}
* {{ change }}
{%- else %}
No relevant code changes.
{%- endfor %}
{% endfor %}
"""

KEYWORDS = ["FEATURE", "BUGFIX", "BUG", "MINOR", "MAJOR", "PATCH", "PERFORMANCE"]


def changelog(project_dir):
    repo = Repo(project_dir)
    tag_commits = {tag.commit: tag for tag in repo.tags}

    current_version = None
    changes = []
    version_changes = OrderedDict()

    for commit in repo.iter_commits():
        if commit in tag_commits:
            version_changes[current_version] = changes
            current_version = tag_commits[commit].name
            changes = []

        message = commit.message

        for keyword in KEYWORDS:
            if message.startswith(keyword):
                if message not in changes:  # don't add dupes
                    changes.append(message)

    return jinja2.Template(CHANGELOG_MD_TEMPLATE).render(
        version_changes=version_changes
    )


def directory_template(
    all_stories, project_dir, story_dir, build_dir, readme=False
):
    return (
        dirtemplate.DirTemplate(project_dir / "docs", build_dir)
        .with_files(
            template_story_jinja2={
                "using/alpha/{0}.md".format(story.info["docs"]): {"story": story}
                for story in all_stories.ordered_by_name()
                if story.info.get("docs") is not None
            }
        )
        .with_vars(
            readme=readme,
            quickstart=all_stories.in_filename(story_dir / "quickstart.story")
            .non_variations()
            .ordered_by_file(),
            include_title=True,
        )
        .with_functions(title=title)
    )


def title(dirfile):
    assert len(dirfile.text().split("---")) >= 3, "{} doesn't have ---".format(dirfile)
    return load(dirfile.text().split("---")[1]).data.get("title", "misc")


def docgen(all_stories, project_dir, story_dir, build_dir):
    """
    Generate markdown documentation.
    """

    docfolder = build_dir / "docs"
    if docfolder.exists():
        docfolder.rmtree(ignore_errors=True)
    docfolder.mkdir()

    directory_template(
        all_stories, project_dir, story_dir, docfolder, readme=False
    ).ensure_built()
    docfolder.joinpath("changelog.md").write_text(changelog(project_dir))
    print("Docs generated")


def readmegen(all_stories, project_dir, story_dir, build_dir, project_name):
    docfolder = build_dir / "readme"
    if docfolder.exists():
        docfolder.rmtree(ignore_errors=True)
    docfolder.mkdir()

    directory_template(
        all_stories, project_dir, story_dir, docfolder, readme=True
    ).ensure_built()

    import re

    text_with_absolute_links = re.sub(
        r"(\[.*?\])\(((?!http).*?)\)",
        r"\g<1>(https://hitchdev.com/{0}/\g<2>)".format(project_name),
        docfolder.joinpath("index.md").text(),
    )

    project_dir.joinpath("CHANGELOG.md").write_text(changelog(project_dir))
    project_dir.joinpath("README.md").write_text(text_with_absolute_links)
    print("README and CHANGELOG generated")



from hitchpylibrarytoolkit.deploy import deploy
from hitchpylibrarytoolkit.docgen import docgen
from hitchpylibrarytoolkit.docgen import readmegen
from hitchpylibrarytoolkit.build import project_build
from hitchpylibrarytoolkit.formatter import reformat
from hitchpylibrarytoolkit.formatter import lint
from hitchpylibrarytoolkit.build import PyLibraryBuild
from hitchstory import StoryCollection
from pathquery import pathquery
from hitchstory import HitchStoryException
from hitchrun import expected


class ToolkitError(Exception):
    pass


class ProjectToolkit(object):
    def __init__(self, project_name, paths):
        self._project_name = project_name
        self._path = paths

    @property
    def current_version(self):
        return self._path.project.joinpath("VERSION").text().rstrip()

    @property
    def build(self):
        return PyLibraryBuild(
            self._project_name,
            self._path,
        )

    def bdd(self, engine, keywords):
        """Run individual story matching key words."""
        self._stories(engine).only_uninherited().shortcut(*keywords).play()

    def regression(self, engine):
        self._stories(engine).only_uninherited().ordered_by_name().play()
        

    def _stories(self, engine):
        return StoryCollection(
            pathquery(self._path.key / "story").ext("story"),
            engine,
        )

    def prepdeploy(self):
        self.regression()
        readmegen(
            self._stories(),
            self._path.project,
            self._path.key,
            self._path.gen,
            self._project_name,
        )

    def deploy(self, version):
        deploy(self._path.project, self._project_name, version)

    def lint(self, exclude=None):
        if exclude is None:
            exclude = "__init__.py"
        python("-m", "flake8")(
            project_dir.joinpath(self._project_name),
            "--max-line-length=100",
            "--exclude={}".format(",".join(exclude)),
            "--ignore=E203,W503",  # Ignore list[expression1 : expression2] failures
        ).run()
        python("-m", "flake8")(
            project_dir.joinpath("hitch", "key.py"),
            "--max-line-length=100",
            "--ignore=E203,W503",
        ).run()

    def reformat(self):
        reformat(self._path.project, self._project_name)

    def docgen(self, engine):
        docgen(self._stories(engine), self._path.project, self._path.key / "story", self._path.gen)

    def readmegen(self, engine):
        readmegen(self._stories(engine), self._path.project, self._path.key / "story", self._path.gen, self._project_name)



from commandlib import python, python_bin

def reformat(project_dir, project_name):
    python_bin.black(project_dir / project_name).run()
    python_bin.black(project_dir / "hitch" / "key.py").run()
    print("Reformat success")
    lint(project_dir, project_name)



# -*- coding: utf-8 -*
from setuptools.command.install import install
from setuptools import find_packages
from setuptools import setup
from sys import version_info, stderr, exit
import codecs
import sys
import os


def read(*parts):
    # intentionally *not* adding an encoding option to open
    # see here: https://github.com/pypa/virtualenv/issues/201#issuecomment-3145690
    return codecs.open(os.path.join(os.path.abspath(os.path.dirname(__file__)), *parts), 'r').read()


setup(name="hitchpylibrarytoolkit",
      version=read('VERSION').replace('\n', ''),
      description="Build, test, documentation, linting, reformatting and specification code for hitch libraries.",
      long_description=read('README.md'),
      classifiers=[
          'Development Status :: 4 - Beta',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Topic :: Text Processing :: Markup',
          'Topic :: Software Development :: Libraries',
          'Natural Language :: English',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.1',
          'Programming Language :: Python :: 3.2',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
      ],
      keywords='yaml',
      author='Colm O\'Connor',
      author_email='colm.oconnor.github@gmail.com',
      url='http://hitchdev.com/hitchpylibrarytoolkit',
      license='MIT',
      install_requires=[
          "hitchstory>=0.12.0",
          "hitchrunpy>=0.6.1",
          "hitchbuildpy>=0.4.5",
          "dirtemplate>=0.1.4",
          "templex>=0.2.0",
          "kaching>=0.4",
          "twine>=0.11.0",
          "flake8>=3.5.0",
          "hitchrun",
          "gitpython",
          "black",
          "ipython",
          "q>=2.6",
      ],
      packages=find_packages(exclude=["tests", "docs", ]),
      package_data={},
      zip_safe=False,
      include_package_data=True,
)
